/*
asc2ppm.c Version 0.2.0
Copyright (C) 2003-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: asc2ppm 19 19 <diagram | pnmtopng >diagram.png */

/* The first parameter is the number of columns 1-19 */
/* The second parameter is the number of rows 1-19 */
/* The output of asc2ppm is a ppm file that is piped to */
/* a program that converts the ppm format to a compressed */
/* graphic image. */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define FNT 19
#define FNTPTH "19"
#undef  FNT
#undef  FNTPTH
#define FNT 27
#define FNTPTH "27"
#define MAXROW (19)
#define FROWLEN (FNT)
#define ROWLEN (FROWLEN*MAXROW)
#define MTXSZ (ROWLEN*ROWLEN)
#define FONTSZ (FROWLEN*FROWLEN)


int eofsw,rowlen;
int maxrow,maxcol;
unsigned char mtx[MTXSZ];

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s rows columns\n", pgm);
   fprintf(stderr,"Where rows is 1 - %d\n", MAXROW);
   fprintf(stderr,"      columns is 1 - %d\n", MAXROW);
   fprintf(stderr,"Example: %s 9 9\n", pgm);
   exit(1);
   } /* putstx */

int getbyte()
   {
   int rdlen;
   unsigned char buf[8];
   rdlen = read(0,buf,1);
   if (rdlen == 1) return(buf[0]);
   else if (!rdlen)
      {
      eofsw = 1;
      return(EOF);
      } /* eof */
   else
      {
      perror("Read error");
      exit(1);
      } /* readerr */
   } /* getbyte */

int getnode(node)
char *node;
   {
   int ch,eolsw;
   char *p,*q;
   eolsw = 0;
   p = node;
   q = p + 16;
   *p = '\0';
   while ((ch = getbyte()) != EOF
      && (ch < 'a' || ch > 'z')
      && ch != '.') ; /* bypass white space */
   if (ch == EOF) return(0);
   if (ch == '\n') eolsw = 1;
   while (p < q && ch > ' ' && ch <= 'z' && ch != EOF)
      {
      *p++ = ch;
      ch = getbyte();
      } /* for each char in node */
   *p = '\0';
   if (ch == '\n') eolsw = 1;
   return(eolsw);
   } /* getnode */

void putbuf(len)
int len;
   {
   int wrtlen;
   wrtlen = write(1,mtx,len);
   if (wrtlen == len) return;
   else
      {
      fprintf(stderr,"Write length %d\n", wrtlen);
      perror("Write error");
      exit(1);
      } /* write err */
   } /* putbuf */

void isrtfont(buf,row,col)
unsigned char *buf;
int row;
int col;
   {
   int i;
   int x,y;
   unsigned char *p,*q,*r;
   p = buf;
   y = row * rowlen * FROWLEN;
   x = col * FROWLEN;
   r = mtx + y + x;
   i = 0;
   while (i < FROWLEN)
      {
      q = p + FROWLEN;
      while (p < q)
	 {
	 *r++ = *p++;
	 } /* for each row in font */
      r = r + rowlen - FROWLEN;
      i++;
      } /* outer loop */
   } /* isrtfont */

void ldfont(hndl,font,row,col)
int hndl;
char *font;
int row;
int col;
   {
   int rdlen;
   unsigned char buf[FONTSZ];
   rdlen = read(hndl,buf,FONTSZ);
   if (rdlen == FONTSZ)
      {
      isrtfont(buf,row,col);
      } /* good read */
   else if (!rdlen)
      {
      fprintf(stderr,"Empty font %s\n", font);
      exit(1);
      } /* empty font */
   else
      {
      fprintf(stderr,"%s\n", font);
      fprintf(stderr,"Read length %d\n", rdlen);
      perror("Read error");
      exit(1);
      } /* read err */
   } /* ldfont */

int clsfont(hndl)
int hndl;
   {
   int stat;
   stat = close(hndl);
   if (hndl < 0)
      {
      perror("Close error");
      exit(1);
      } /* close err */
   } /* clsfont */

int opnfont(font,row,col)
char *font;
int row;
int col;
   {
   int hndl,rdlen;
   char fname[128];
   sprintf(fname,"%s/%s.fnt", FNTPTH, font);
   hndl = open(fname,O_RDONLY);
   if (hndl >= 0)
      {
      ldfont(hndl,font,row,col);
      clsfont(hndl);
      } /* if good open */
   else
      {
      fprintf(stderr,"%s\n", font);
      perror("Open error");
      exit(1);
      } /* open err */
   } /* opnfont */

int main(argc,argv)
int argc;
char **argv;
   {
   int eolsw,row,col;
   unsigned char *p,*q;
   char node[16];
   char str[128];

   if (argc != 3) putstx(*argv);
   maxcol = atoi(*(argv+1));
   if (maxcol < 1 || maxcol > MAXROW)
      {
      fprintf(stderr,"Invalid columns %s\n", *(argv+1));
      putstx(*argv);
      } /* bad maxcol */
   maxrow = atoi(*(argv+2));
   if (maxrow < 1 || maxrow > MAXROW)
      {
      fprintf(stderr,"Invalid rows %s\n", *(argv+1));
      putstx(*argv);
      } /* bad maxrow */
   rowlen = maxcol * FROWLEN;

   /* initialize matrix */
   p = mtx;
   q = mtx + MTXSZ;
   while (p < q) *p++ = (unsigned char) 255;

   row = col = eofsw = 0;
   while (!eofsw)
      {
      eolsw = getnode(node,&row,&col);
      if (eofsw) break;
      opnfont(node,row,col);
      col++;
      if (eolsw)
	 {
	 row++;
	 col = 0;
	 } /* if end of row */
      } /* node loop */
   sprintf(str,"P5\n%d %d\n255\n",
      rowlen, maxrow*FROWLEN);
   printf("%s", str);
   fflush(stdout);         /* synchronize output */
   putbuf(rowlen*maxrow*FROWLEN);
   return(0);
   } /* main */
